import json
import requests

def get_receiver(base_url):
    receiver_url = f'{base_url}/data/receiver.json'

    response = requests.get(receiver_url)
    json_obj = json.loads(response.content)

    return json_obj



import math
import json
import requests
from adsb_tools.utils import requests_utils

EARTH_RADIUS_KM = 6371.0

def calculate_distance(lat1, lon1, lat2, lon2):
    """
    Calculates the great-circle distance between two points on the Earth's surface
    using the Haversine formula.

    Args:
        lat1 (float): Latitude of the first point in degrees
        lon1 (float): Longitude of the first point in degrees
        lat2 (float): Latitude of the second point in degrees
        lon2 (float): Longitude of the second point in degrees

    Returns:
        float: Distance between the two points in kilometers
    """

    # Convert coordinates to radians
    lat1, lon1, lat2, lon2 = map(math.radians, [lat1, lon1, lat2, lon2])

    # Haversine formula
    dlat = lat2 - lat1
    dlon = lon2 - lon1
    a = math.sin(dlat/2)**2 + math.cos(lat1) * math.cos(lat2) * math.sin(dlon/2)**2
    c = 2 * math.atan2(math.sqrt(a), math.sqrt(1-a))
    distance = EARTH_RADIUS_KM * c

    return distance


def get_direction(base_lat, base_lon, dest_lat, dest_lon):
    """Calculate the direction from one coordinate to another."""
    # Calculate the difference between the latitudes and longitudes
    lat_diff = dest_lat - base_lat
    lon_diff = dest_lon - base_lon

    # Calculate the angle between the two points in radians
    angle = math.atan2(lon_diff, lat_diff)

    # Convert the angle from radians to degrees
    degrees = math.degrees(angle)

    # Convert the angle to a compass direction
    directions = ['N', 'NE', 'E', 'SE', 'S', 'SW', 'W', 'NW']
    idx = round(degrees / (360.0 / len(directions))) % len(directions)
    direction = directions[idx]

    return degrees, direction


def add_aircraft_options(aircraft_list, base_lat, base_lon):
    """
    Adds additional data to aircraft based on aircraft properties
    """
    aircraft_list_with_options = []
    for aircraft in aircraft_list:
        new_aircraft = aircraft.copy()
        aircraft_lat = new_aircraft['lat']
        aircraft_lon = new_aircraft['lon']

        distance = calculate_distance(base_lat, base_lon, aircraft_lat, aircraft_lon)
        degrees, direction = get_direction(base_lat, base_lon, aircraft_lat, aircraft_lon)

        new_aircraft['distance'] = distance
        new_aircraft['degrees'] = degrees
        new_aircraft['direction'] = direction
        new_aircraft['icao'] = aircraft['hex']

        aircraft_list_with_options.append(new_aircraft)

    return sorted(aircraft_list_with_options, key=lambda x: x["distance"])


def get_aircraft_image(aircraft_hex):
    """
    Gets the image of an aircraft, given its hex. If no image found, return
    empty dictionary.
    """
    headers = {
        'User-Agent': 'My Unique User Agent'
    }
    planespotter_url = f'https://api.planespotters.net/pub/photos/hex/{aircraft_hex}'
    response = requests.get(planespotter_url, headers=headers)
    json_obj = json.loads(response.content)

    image = {}
    if (json_obj['photos'] and json_obj['photos'][0] and bool(json_obj['photos'][0])):
        photo_attributes = json_obj['photos'][0]
        thumbnail = photo_attributes['thumbnail']
        thumbnail_large = photo_attributes['thumbnail_large']
        target_photo = thumbnail if thumbnail_large is None else thumbnail_large
        image = {
            'height': target_photo['size']['height'],
            'width': target_photo['size']['width'],
            'src': target_photo['src'],
            'url': planespotter_url
        }

    return image


def get_hex_db_flight(icao_24):
    """
    Get values from HexDB
    """
    hex_db_url = f'https://hexdb.io/api/v1/aircraft/{icao_24}'
    hex_db_result = requests_utils.call_url(hex_db_url)
    hex_db_obj = json.loads(hex_db_result.content)

    return hex_db_obj


def get_adsb_db_flight(icao_24):
    """
    Get values from ADSB DB
    """
    adsb_db_url = f'https://api.adsbdb.com/v0/aircraft/{icao_24}'
    adsb_db_result = requests_utils.call_url(adsb_db_url)
    adsb_db_obj = json.loads(adsb_db_result.content)

    return adsb_db_obj


def get_aircraft(base_url, filter_aircraft = True):
    """
    Get the aircraft messages and returns as list
    """
    receiver_url = f'{base_url}/data/aircraft.json'

    response = requests_utils.call_url(receiver_url)
    json_obj = json.loads(response.content)
    result = json_obj

    if (filter_aircraft):
        result = [
            d for d in json_obj['aircraft']
            if "lat" in d and "lon" in d and d["lat"] is not None and d["lon"] is not None
        ]

    return result



import requests

def call_url(url: str, timeout: int = 5) -> requests.Response:
    """
    Sends an HTTP GET request to the specified URL with the given timeout.

    Parameters:
        url (str): The URL to send the request to.
        timeout (int): The maximum number of seconds to wait for a response.

    Returns:
        requests.Response: A `Response` object containing information about the response from the server,
            or `None` if an error occurs.
    """
    try:
        print(f"{url}: attempting request...")
        response = requests.get(url=url, timeout=timeout)
        response.raise_for_status()
    except requests.exceptions.Timeout:
        print(f"{url}: timeout error occurred.")
    except requests.exceptions.HTTPError as err:
        print(f"{url}: HTTP error occurred: {err}")
    except requests.exceptions.RequestException as err:
        print(f"{url}: An error occurred: {err}")
    finally:
        print(f'{url}: request completed.')
    return response


import json
import pytz
from datetime import datetime, timezone
from adsb_tools.utils import requests_utils

def get_current_event(events):
    """
    Given a list of events that have start time and end time, return the event
    that matches the interval for the current time.
    """
    current_time = datetime.now(timezone.utc)

    for event in events:
        start_time = event['startTime']
        end_time = event['endTime']
        if isinstance(start_time, str):
            start_time = datetime.fromisoformat(start_time)
        if isinstance(end_time, str):
            end_time = datetime.fromisoformat(end_time)
        if start_time.tzinfo is None:
            start_time = start_time.replace(tzinfo=timezone.utc)
        if end_time.tzinfo is None:
            end_time = end_time.replace(tzinfo=timezone.utc)
        if start_time <= current_time <= end_time:
            return event

    return None


def get_localized_time(date_time, local_timezone, date_time_format = "%-I:%M%p"):
    """
    takes a datetime object and returns a human-readable local time
    """
    formatted_time = datetime.fromisoformat(date_time)
    local_time = formatted_time.astimezone(local_timezone)
    local_formatted_time = local_time.strftime(date_time_format)
    return local_formatted_time


def get_grid_data(latitude, longitude):
    """
    Gets weather grid data which includes forecast url and local timezone
    """
    grid_url = f'https://api.weather.gov/points/{latitude},{longitude}'
    grid_response = requests_utils.call_url(grid_url)

    forcast_hourly_url = None
    local_timezone_name = None
    if (grid_response):
        grid_json_obj = json.loads(grid_response.content)
        grid_properties =  grid_json_obj['properties']
        forcast_hourly_url = grid_properties['forecastHourly']
        local_timezone_name = grid_properties['timeZone']

    return forcast_hourly_url, local_timezone_name


def get_weather_data(forcast_hourly_url, local_timezone_name):
    """
    Get the current weather data from a specified URL and return as a dictionary
    """
    local_timezone_obj = pytz.timezone(local_timezone_name)

    weather_response = requests_utils.call_url(forcast_hourly_url)
    weather_json_obj = json.loads(weather_response.content)

    current_weather = get_current_event(weather_json_obj['properties']['periods'])

    local_formatted_start_time = get_localized_time(current_weather['startTime'], local_timezone_obj)
    local_formatted_end_time = get_localized_time(current_weather['endTime'], local_timezone_obj)

    weather_report = {
        'startTime': local_formatted_start_time,
        'endTime': local_formatted_end_time,
        'shortForecast': current_weather['shortForecast'],
        'temperature': current_weather['temperature'],
        'relativeHumidity': current_weather['relativeHumidity']['value'],
        'windDirection': current_weather['windDirection'],
        'windSpeed': current_weather['windSpeed'],
        'precipitation': current_weather['probabilityOfPrecipitation']['value'],
        'period': current_weather,
        'forcast_hourly_url': forcast_hourly_url
    }

    return weather_report



from setuptools import setup

# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='adsb_tools',
    version='0.1.21',
    packages=['adsb_tools'],
    python_requires='>=3.7',
    install_requires=[
        'requests',
        'pytz',
        'timezonefinder'
    ],
    author='herereadthis',
    author_email='herereadthis.github@gmail.com',
    description='Various tools for handling ADS-B data coming from Dump1090 messages',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/herereadthis/adsb_tools',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)



from timezonefinder import TimezoneFinder

def get_timezone_name(latitude, longitude):
    """
    Gets the name of the timezone based on coordinates
    """
    tf = TimezoneFinder()
    timezone_name = tf.timezone_at(lat=latitude, lng=longitude)

    return timezone_name